﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Phidget22;
using Phidget22.ExampleUtils;

namespace Dictionary_Example {
	public partial class Form1 : Form {
		CommandLineOpen openArgs;
		Phidget22.Dictionary openCh = null;
		Phidget22.Dictionary dict = null;
		private ErrorEventBox errorBox;
		AddEntry addEntryForm;

		public Form1() {
			InitializeComponent();
			openArgs = new CommandLineOpen(this);
		}

		public Form1(Phidget22.Dictionary ch) {
			InitializeComponent();
			openCh = ch;
		}

		private void Form1_Load(object sender, EventArgs e) {

			errorBox = new ErrorEventBox(this);

			dict = openCh ?? openArgs.makeChannel<Phidget22.Dictionary>();

			//register the various event handlers
			dict.Attach += dict_attach;
			dict.Detach += dict_detach;
			dict.Error += dict_error;
			dict.KeyValueUpdate += dict_update;
			dict.KeyAdd += dict_add;
			dict.KeyRemove += dict_remove;

			try {
				dict.Open();
			} catch (PhidgetException ex) { errorBox.addMessage("Error opening device: " + ex.Message); }
		}

		private void Form1_FormClosing(object sender, FormClosingEventArgs e) {

			dict.Attach -= dict_attach;
			dict.Detach -= dict_detach;
			dict.Error -= dict_error;
			dict.KeyValueUpdate -= dict_update;
			dict.KeyAdd -= dict_add;
			dict.KeyRemove -= dict_remove;
			dict.Close();

			if (addEntryForm != null)
				addEntryForm.Close();
		}

		void dict_attach(object sender, Phidget22.Events.AttachEventArgs e) {
			Phidget22.Dictionary attachedDictionary = (Phidget22.Dictionary)sender;
			dictSerialTxt.Text = attachedDictionary.DeviceSerialNumber.ToString();
			dictTxt.Text = attachedDictionary.DeviceLabel;
			dictionaryTable.Enabled = true;
			attachedChk.Checked = true;
			addKey.Enabled = true;
			removeKey.Enabled = true;

			try {
				//get a list of the dictionary contents
				String[] keys = dict.Scan(null); //scan returns a limited number of keys so for large dictionaries you may have to run scan multiple times.  null will start at the beginning

				while (keys.Length > 0) {

					//populate the form with the dictionary contents
					foreach (String element in keys)
						dictionaryTable.Rows.Add(element, dict.Get(element));

					keys = dict.Scan(keys[keys.Length - 1]); //now we scan beginning at the key which was the last key we previously scanned
				}
			} catch (Exception ex) { errorBox.addMessage("Error reading dictionary contents: " + ex.Message); }
		}

		void dict_detach(object sender, Phidget22.Events.DetachEventArgs e) {
			dictSerialTxt.Clear();
			dictTxt.Clear();
			attachedChk.Checked = false;
			dictionaryTable.Rows.Clear();
			dictionaryTable.Enabled = false;
			addKey.Enabled = false;
			removeKey.Enabled = false;
		}

		void dict_error(object sender, Phidget22.Events.ErrorEventArgs e) {
			errorBox.addMessage(e.Description);
		}

		void dict_update(object sender, Phidget22.Events.DictionaryKeyValueUpdateEventArgs e) {
			foreach (DataGridViewRow row in dictionaryTable.Rows) {//go through the table
				if (row.Cells[0].Value.ToString() == e.Key) {//find the row with the matching key
					row.Cells[1].Value = e.Value; //update with the new value
					break; //stop looking
				}
			}
		}

		void dict_add(object sender, Phidget22.Events.DictionaryKeyAddEventArgs e) {//a new entry had been added to the dictionary, update the form to reflect this
			dictionaryTable.Rows.Add(e.Key, e.Value);
		}

		void dict_remove(object sender, Phidget22.Events.DictionaryKeyRemoveEventArgs e) {//an entry has been removed from the dictionary, update the form to reflect this
			keyRemoved(e.Key);
		}

		void keyRemoved(string key) {
			int indexToDelete = -1;
			foreach (DataGridViewRow row in dictionaryTable.Rows) {//scan the table
				if (row.Cells[0].Value.ToString() == key) {//find the row with the deleted key
					indexToDelete = row.Index;
					break;
				}
			}
			if (indexToDelete >= 0)//delete the row with the deleted key
				dictionaryTable.Rows.RemoveAt(indexToDelete);
		}

		private void addKey_Click(object sender, EventArgs e) {
			addEntryForm = new AddEntry();
			var result = addEntryForm.ShowDialog();
			if (result == DialogResult.OK) {
				try {
					dict.Add(addEntryForm.key, addEntryForm.value);
					dictionaryTable.Rows.Add(addEntryForm.key, addEntryForm.value);
				} catch (PhidgetException ex) { errorBox.addMessage("Error adding key: " + ex.Message); }
			}
		}

		private void removeKey_Click(object sender, EventArgs e) {
			foreach (DataGridViewRow selection in dictionaryTable.SelectedRows) {
				try {
					dict.Remove(selection.Cells[0].Value.ToString());
					keyRemoved(selection.Cells[0].Value.ToString());
				} catch (PhidgetException ex) { errorBox.addMessage("Error removing key: " + ex.Message); }
			}
		}

		private void button1_Click(object sender, EventArgs e) {
			addEntryForm = new AddEntry(dictionaryTable.SelectedRows[0].Cells[0].Value.ToString()); //fill key we are trying to change
			var result = addEntryForm.ShowDialog();
			if (result == DialogResult.OK) {
				try {
					dictionaryTable.SelectedRows[0].Cells[1].Value = addEntryForm.value;
					dict.Update(addEntryForm.key, addEntryForm.value);
				} catch (PhidgetException ex) { errorBox.addMessage("Error updating key/value pair: " + ex.Message); }
			}
		}

		private void dictionaryTable_SelectionChanged(object sender, EventArgs e) {
			if (dictionaryTable.SelectedRows.Count == 1)
				updateBtn.Enabled = true;
			else
				updateBtn.Enabled = false;
		}
	}
}
